@@ -4,8 +4,7 @@ package Frost::Burial;
 #
 use Moose;
 
-#	use DB_File 1.820;
-use BerkeleyDB 0.43;
+use DB_File 1.820;
 
 #s	SPEEDFIX: All checks removed: We will never use this module stand-alone...
 #s	Update: Re-introduce some checks...
@@ -19,7 +18,7 @@ use Frost::Util;
 
 #	CLASS VARS
 #
-our $VERSION	= 0.69;
+our $VERSION	= 0.68;
 our $AUTHORITY	= 'cpan:ERNESTO';
 
 #	CLASS METHODS
@@ -29,13 +28,13 @@ sub suffix { die 'Abstract method' }
 #	PUBLIC ATTRIBUTES
 #
 has data_root	=> ( isa => 'Frost::FilePathMustExist',	is => 'ro',								required => true,		);
-has classname	=> ( isa => 'ClassName',						is => 'ro',								required => true,		);
-has slotname	=> ( isa => 'Str',								is => 'ro',								required => true,		);
+has classname	=> ( isa => 'ClassName',			is => 'ro',								required => true,		);
+has slotname	=> ( isa => 'Str',					is => 'ro',								required => true,		);
 
-has filename	=> ( isa => 'Frost::FilePath',	is => 'ro',	init_arg => undef,	lazy_build => true,	);
+has filename	=> ( isa => 'Frost::FilePath',				is => 'ro',	init_arg => undef,	lazy_build => true,	);
 has numeric		=> ( isa => 'Bool', 					is => 'ro',	init_arg => undef,	lazy_build => true,	);
 has unique		=> ( isa => 'Bool',					is => 'ro',	init_arg => undef,	lazy_build => true,	);
-has cachesize	=> ( isa => 'Frost::Natural',		is => 'ro',			default => DEFAULT_CACHESIZE,			);
+has cachesize	=> ( isa => 'Frost::Natural',				is => 'ro',			default => DEFAULT_CACHESIZE,			);
 
 #	PRIVATE ATTRIBUTES
 #
@@ -53,13 +52,6 @@ has _dbm_object	=>
 	default		=> sub { undef },
 );
 
-has _dbm_cursor	=>
-(
-	is				=> 'rw',
-	isa			=> 'Undef | Frost::DBM_Cursor',
-	default		=> sub { undef },
-);
-
 #	CONSTRUCTORS
 #
 sub _build_filename
@@ -97,107 +89,42 @@ sub BUILDARGS
 
 #	DESTRUCTORS
 #
-#	Autosave does SOMETIMES NOT work!
-#
-#	Frost::Burial::DEMOLISH ( Frost::Cemetery=HASH(0x9f432e0) 1 )
-#	Frost::Burial::close
-#	$is_closed = 0;
-#	$filename = "/tmp/frost/Point/x.cem";
-#	$in_global_destruction = 1;
-#	Frost::Burial::save
-#	$is_closed = 0;
-#	$filename = "/tmp/frost/Point/x.cem";
-#	$in_global_destruction = 1;
-#	Frost::Burial::save B4 sync...
-#		(in cleanup) BerkeleyDB Aborting: Database is already closed at ../lib/Frost/Burial.pm line 217 during global destruction.
-#	 at ../lib/Frost/Burial.pm line 217
-#		Frost::Burial::save('Frost::Cemetery=HASH(0x9f432e0)', 1) called at ../lib/Frost/Burial.pm line 190
-#		Frost::Burial::close('Frost::Cemetery=HASH(0x9f432e0)', 1) called at ../lib/Frost/Burial.pm line 104
-#		Frost::Burial::DEMOLISH('Frost::Cemetery=HASH(0x9f432e0)', 1) called at generated method (unknown origin) line 7
-#		Class::MOP::Method::Generated::__ANON__() called at /usr/local/share/perl/5.10.0/Try/Tiny.pm line 76
-#		eval {...} called at /usr/local/share/perl/5.10.0/Try/Tiny.pm line 67
-#		Try::Tiny::try('CODE(0xa0f03d8)', 'Try::Tiny::Catch=REF(0x9f63628)') called at generated method (unknown origin) line 8
-#		Frost::Cemetery::DESTROY('Frost::Cemetery=HASH(0x9f432e0)') called at 000_moose_recipes/moose_cookbook_basics_recipe1.t line 0
-#		eval {...} called at 000_moose_recipes/moose_cookbook_basics_recipe1.t line 0
-#	 at generated method (unknown origin) line 8
-#		Class::MOP::Method::Generated::__ANON__('BerkeleyDB Aborting: Database is already closed at ../lib/Fro...') called at /usr/local/share/perl/5.10.0/Try/Tiny.pm line 98
-#		Try::Tiny::try('CODE(0xa0f03d8)', 'Try::Tiny::Catch=REF(0x9f63628)') called at generated method (unknown origin) line 8
-#		Frost::Cemetery::DESTROY('Frost::Cemetery=HASH(0x9f432e0)') called at 000_moose_recipes/moose_cookbook_basics_recipe1.t line 0
-#		eval {...} called at 000_moose_recipes/moose_cookbook_basics_recipe1.t line 0
-#
-#	sub DEMOLISH
-#	{
-#		print STDERR __PACKAGE__ . "::DEMOLISH ( @_ )\n";
-#
-#		$_[0]->close ( $_[1] || 0 );		#	in global destruction...
-#
-#		print STDERR __PACKAGE__ . "::DEMOLISH done\n\n";
-#	}
+sub DEMOLISH	{ $_[0]->close(); }
 
 #	PUBLIC METHODS
 #
 sub is_open			{ ( defined $_[0]->{_dbm_object} ) ? true : false }		#	return 'real' boolean
 sub is_closed		{ ( defined $_[0]->{_dbm_object} ) ? false : true }
 
-#	sub open
-#	{
-#		#IS_DEBUG and DEBUG "( @_ )";
-#
-#		my ( $self )	= @_;
-#
-#		return true		if $self->is_open;
-#
-#		my %dbm_hash;
-#
-#		my $filename	= $self->filename;
-#		my $flags		= O_RDWR|O_CREAT;
-#		my $mode			= 0700;
-#		my $info			= new DB_File::BTREEINFO;
-#
-#		$info->{'flags'} 		= R_DUP						unless $self->unique;
-#		$info->{'cachesize'}	= $self->cachesize;
-#	#	$info->{'compare'}	= \&_numeric_compare		if $self->numeric;
-#	#
-#		my $dbm_object		= tie %dbm_hash, "DB_File", $filename, $flags, $mode, $info
-#												or die "Cannot open $filename: $!\n";
-#
-#		if ( $self->numeric )
-#		{
-#			no warnings;		#	$_ might be undef !!!
-#
-#			$dbm_object->filter_fetch_key  ( sub { $_ = unpack	("i", $_) } );
-#			$dbm_object->filter_store_key  ( sub { $_ = pack	("i", $_) } );
-#		}
-#
-#		$self->_dbm_hash		( \%dbm_hash );
-#		$self->_dbm_object	( $dbm_object );
-#
-#		return true;
-#	}
-
 sub open
 {
-	my ( $self )	= @_;
+	#IS_DEBUG and DEBUG "( @_ )";
 
-	IS_DEBUG and DEBUG Dump [ $self->is_open, $self->filename ], [qw( is_open filename )];
+	my ( $self )	= @_;
 
 	return true		if $self->is_open;
 
 	my %dbm_hash;
 
 	my $filename	= $self->filename;
+	my $flags		= O_RDWR|O_CREAT;
+	my $mode			= 0700;
+	my $info			= new DB_File::BTREEINFO;
 
-	my $info			= {};
+	$info->{'flags'} 		= R_DUP						unless $self->unique;
+	$info->{'cachesize'}	= $self->cachesize;
+#	$info->{'compare'}	= \&_numeric_compare		if $self->numeric;
+#
+	my $dbm_object		= tie %dbm_hash, "DB_File", $filename, $flags, $mode, $info
+											or die "Cannot open $filename: $!\n";
 
-	$info->{'-Filename'} 	= $filename;
-	$info->{'-Flags'} 		= DB_CREATE;
-	$info->{'-Mode'} 			= 0700;
-	$info->{'-Property'}		= DB_DUP								unless $self->unique;
-	$info->{'-Cachesize'}	= $self->cachesize;
-	$info->{'-Compare'}		= sub { $_[0] <=> $_[1] }		if $self->numeric;
+	if ( $self->numeric )
+	{
+		no warnings;		#	$_ might be undef !!!
 
-	my $dbm_object	= tie %dbm_hash, "BerkeleyDB::Btree", %$info
-											or die "Cannot open $filename: $! $BerkeleyDB::Error\n" ;
+		$dbm_object->filter_fetch_key  ( sub { $_ = unpack	("i", $_) } );
+		$dbm_object->filter_store_key  ( sub { $_ = pack	("i", $_) } );
+	}
 
 	$self->_dbm_hash		( \%dbm_hash );
 	$self->_dbm_object	( $dbm_object );
@@ -207,26 +134,19 @@ sub open
 
 sub close
 {
-	#IS_DEBUG and DEBUG "( @_ )";
-
-	my ( $self, $igd )	= @_;
-
-	#print STDERR __PACKAGE__ . "::close", Dump [ $self->is_closed, $self->filename, $igd ], [qw( is_closed filename in_global_destruction )];
+	my ( $self )	= @_;
 
 	return true		if $self->is_closed;
 
-	$self->save ( $igd );
+	$self->save();
 
 	my $dbm_hash	= $self->_dbm_hash;
 
-	$self->_dbm_cursor	( undef );		#	release cursor
 	$self->_dbm_hash		( undef );
 	$self->_dbm_object	( undef );
 
 	untie %$dbm_hash;
 
-	#print STDERR __PACKAGE__ . "::close done\n\n";
-
 	return true;
 }
 
@@ -234,22 +154,14 @@ sub save
 {
 	#IS_DEBUG and DEBUG "( @_ )";
 
-	my ( $self, $igd )	= @_;
-
-	#print STDERR __PACKAGE__ . "::save", Dump [ $self->is_closed, $self->filename, $igd ], [qw( is_closed filename in_global_destruction )];
+	my ( $self )	= @_;
 
 	return true			if $self->is_closed;
 
-	#print STDERR __PACKAGE__ . "::save B4 sync...\n";
-
-	my $status	= $self->_dbm_object->db_sync();
-
-	#print STDERR __PACKAGE__ . "::save AF sync... status=", ( $status ? $status : 'OK' ), "\n";
+	my $status	= $self->_dbm_object->sync();
 
 	return false		if $status;
 
-	#print STDERR __PACKAGE__ . "::save done\n\n";
-
 	return true;
 }
 
@@ -423,18 +335,7 @@ sub first
 
 	my $reset_key	= undef;
 
-#	from DB_File-1.820\DB_File.xs
-#
-#define R_CURSOR        DB_SET_RANGE
-#define R_FIRST         DB_FIRST
-#define R_IAFTER        DB_AFTER
-#define R_IBEFORE       DB_BEFORE
-#define R_LAST          DB_LAST
-#define R_NEXT          DB_NEXT
-#define R_NOOVERWRITE   DB_NOOVERWRITE
-#define R_PREV          DB_PREV
-
-	@kv			= $self->_seq ( $reset_key, DB_FIRST()	);			#	reset cursor...
+	@kv			= $self->_seq ( $reset_key, R_FIRST()	);			#	reset cursor...
 
 	#IS_DEBUG and DEBUG 'RESET', Dump [ \@kv ], [qw( kv )];
 
@@ -450,7 +351,7 @@ sub first
 
 	#	we need the partial match here...
 	#
-	@kv			= $self->_seq ( $key, DB_SET_RANGE() );
+	@kv			= $self->_seq ( $key, R_CURSOR() );
 
 	$self->_match_key ( $key, \@kv, \$essence, $match );
 
@@ -474,7 +375,7 @@ sub last
 
 	my $reset_key	= undef;
 
-	@kv			= $self->_seq ( $reset_key, DB_LAST()	);			#	reset cursor...
+	@kv			= $self->_seq ( $reset_key, R_LAST()	);			#	reset cursor...
 
 	#IS_DEBUG and DEBUG 'RESET', Dump [ \@kv ], [qw( kv )];
 
@@ -490,7 +391,7 @@ sub last
 
 	#	we need the partial match here...
 	#
-	@kv			= $self->_seq ( $key, DB_SET_RANGE() );		#	matches FIRST entry!
+	@kv			= $self->_seq ( $key, R_CURSOR() );		#	matches FIRST entry!
 
 	$self->_match_key ( $key, \@kv, \$essence, $match );
 
@@ -512,7 +413,7 @@ sub next
 	my @kv		= ();
 	my $essence	= '';
 
-	@kv			= $self->_seq ( $key, DB_NEXT()	);
+	@kv			= $self->_seq ( $key, R_NEXT()	);
 
 	unless ( defined $key )
 	{
@@ -542,7 +443,7 @@ sub prev
 	my @kv		= ();
 	my $essence	= '';
 
-	@kv			= $self->_seq ( $key, DB_PREV()	);
+	@kv			= $self->_seq ( $key, R_PREV()	);
 
 	unless ( defined $key )
 	{
@@ -573,42 +474,21 @@ sub _seq
 		#
 		#	find is called from outside, i.e. via Frost::Asylum::find
 		#
-#		return ( wantarray ? () : '' )		unless $self->_check_key ( $key );
-#
-		unless ( $self->_check_key ( $key ) )
-		{
-			$self->_dbm_cursor ( undef );		#	release cursor
-
-			return ( wantarray ? () : '' );
-		}
+		return ( wantarray ? () : '' )		unless $self->_check_key ( $key );
 	}
 
 	my ( $seq_key, $seq_value, $status );
 
-	$seq_key		= defined $key ? $key : '';	#	do not change param!		must be defined for BerkeleyDB!
+	$seq_key		= $key;	#	do not change param!
 	$seq_value	= '';
 
-#	$status	= $self->_dbm_object->seq ( $seq_key, $seq_value, $cursor );
+	$status	= $self->_dbm_object->seq ( $seq_key, $seq_value, $cursor );
 
-	#	reuse cursor or make new one
-	#
-	#	document/test this... always: ->first ... -> next until empty etc.
-	#
-	$self->_dbm_cursor ( $self->_dbm_object->db_cursor() )		unless $self->_dbm_cursor;
-
-	IS_DEBUG and DEBUG 'CURSOR B4', Dump [ $seq_key, $seq_value, $status, $self->_dbm_cursor ], [qw( seq_key seq_value status db_cursor)];
-
-	$status			= $self->_dbm_cursor->c_get ( $seq_key, $seq_value, $cursor );
-
-#	#IS_DEBUG and DEBUG 'STATUS', Dump [ $seq_key, $seq_value, $status ], [qw( seq_key seq_value status )];
-	IS_DEBUG and DEBUG 'CURSOR AF', Dump [ $seq_key, $seq_value, $status, $self->_dbm_cursor ], [qw( seq_key seq_value status db_cursor)];
-
-	$self->_dbm_cursor ( undef )			if $status;									#	release cursor
+	#IS_DEBUG and DEBUG 'STATUS', Dump [ $seq_key, $seq_value, $status ], [qw( seq_key seq_value status )];
 
 	return ( wantarray ? () : '' )		if $status;
 
-	$self->_dbm_cursor ( undef )			unless defined $seq_value;				#	release cursor
-	$seq_value	= ''							unless defined $seq_value;
+	$seq_value	= ''			unless defined $seq_value;
 
 	return wantarray ? ( $seq_key, $seq_value ) : $seq_value;
 }
@@ -759,9 +639,7 @@ Abstract method - overwrite.
 
 =head1 DESTRUCTORS
 
-=head2 NO Autosave!
-
-Not saying $ASYLUM->close at the end of a program might cause loss of data!
+=head2 DEMOLISH
 
 =head1 PUBLIC METHODS
 
@@ -859,4 +737,3 @@ This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
 =cut
-
